@@ -214,7 +214,10 @@ impl TermOutHandler {
                         make_request(
                             self.config.replay_client.as_ref().unwrap(),
                             resp.url(),
+                            resp.method().as_str(),
+                            None,
                             self.config.output_level,
+                            &self.config,
                             tx_stats.clone(),
                         )
                         .await
