@@ -7,7 +7,7 @@ use crate::{
     skip_fail,
     utils::{fmt_err, logged_request},
     Command::AddFilter,
-    SIMILARITY_THRESHOLD,
+    DEFAULT_METHOD, SIMILARITY_THRESHOLD,
 };
 use anyhow::Result;
 use fuzzyhash::FuzzyHash;
@@ -72,10 +72,17 @@ pub async fn initialize(handles: Arc<Handles>) -> Result<()> {
         let url = skip_fail!(Url::parse(similarity_filter));
 
         // attempt to request the given url
-        let resp = skip_fail!(logged_request(&url, handles.clone()).await);
+        let resp = skip_fail!(logged_request(&url, DEFAULT_METHOD, None, handles.clone()).await);
 
         // if successful, create a filter based on the response's body
-        let fr = FeroxResponse::from(resp, true, handles.config.output_level).await;
+        let fr = FeroxResponse::from(
+            resp,
+            similarity_filter,
+            DEFAULT_METHOD,
+            true,
+            handles.config.output_level,
+        )
+        .await;
 
         // hash the response body and store the resulting hash in the filter object
         let hash = FuzzyHash::new(&fr.text()).to_string();
