@@ -13,6 +13,7 @@ use crate::{
     skip_fail,
     url::FeroxUrl,
     utils::{ferox_print, fmt_err, logged_request, status_colorizer},
+    DEFAULT_METHOD,
 };
 
 /// length of a standard UUID, used when determining wildcard responses
@@ -20,10 +21,11 @@ const UUID_LENGTH: u64 = 32;
 
 /// wrapper around ugly string formatting
 macro_rules! format_template {
-    ($template:expr, $length:expr) => {
+    ($template:expr, $method:expr, $length:expr) => {
         format!(
             $template,
             status_colorizer("WLD"),
+            $method,
             "-",
             "-",
             "-",
@@ -89,55 +91,68 @@ impl HeuristicTests {
             return Ok(0);
         }
 
+        let data = match self.handles.config.data.is_empty() {
+            true => None,
+            false => Some(self.handles.config.data.as_slice()),
+        };
+
         let ferox_url = FeroxUrl::from_string(target_url, self.handles.clone());
 
-        let ferox_response = self.make_wildcard_request(&ferox_url, 1).await?;
+        for method in self.handles.config.methods.iter() {
+            let ferox_response = self
+                .make_wildcard_request(&ferox_url, method.as_str(), data, 1)
+                .await?;
 
-        // found a wildcard response
-        let mut wildcard = WildcardFilter::new(self.handles.config.dont_filter);
+            // found a wildcard response
+            let mut wildcard = WildcardFilter::new(self.handles.config.dont_filter);
 
-        let wc_length = ferox_response.content_length();
+            let wc_length = ferox_response.content_length();
+
+            if wc_length == 0 {
+                log::trace!("exit: wildcard_test -> 1");
+                self.send_filter(wildcard)?;
+                return Ok(1);
+            }
+
+            // content length of wildcard is non-zero, perform additional tests:
+            //   make a second request, with a known-sized (64) longer request
+            let resp_two = self
+                .make_wildcard_request(&ferox_url, method.as_str(), data, 3)
+                .await?;
+
+            let wc2_length = resp_two.content_length();
+
+            wildcard.method = resp_two.method().as_str().to_owned();
+
+            if wc2_length == wc_length + (UUID_LENGTH * 2) {
+                // second length is what we'd expect to see if the requested url is
+                // reflected in the response along with some static content; aka custom 404
+                let url_len = ferox_url.path_length()?;
+
+                wildcard.dynamic = wc_length - url_len;
+
+                if matches!(
+                    self.handles.config.output_level,
+                    OutputLevel::Default | OutputLevel::Quiet
+                ) {
+                    let msg = format_template!("{} {:>8} {:>9} {:>9} {:>9} Wildcard response is dynamic; {} ({} + url length) responses; toggle this behavior by using {}\n", method, wildcard.dynamic);
+                    ferox_print(&msg, &PROGRESS_PRINTER);
+                }
+            } else if wc_length == wc2_length {
+                wildcard.size = wc_length;
+
+                if matches!(
+                    self.handles.config.output_level,
+                    OutputLevel::Default | OutputLevel::Quiet
+                ) {
+                    let msg = format_template!("{} {:>8} {:>9} {:>9} {:>9} Wildcard response is static; {} {} responses; toggle this behavior by using {}\n", method, wildcard.size);
+                    ferox_print(&msg, &PROGRESS_PRINTER);
+                }
+            }
 
-        if wc_length == 0 {
-            log::trace!("exit: wildcard_test -> 1");
             self.send_filter(wildcard)?;
-            return Ok(1);
         }
 
-        // content length of wildcard is non-zero, perform additional tests:
-        //   make a second request, with a known-sized (64) longer request
-        let resp_two = self.make_wildcard_request(&ferox_url, 3).await?;
-
-        let wc2_length = resp_two.content_length();
-
-        if wc2_length == wc_length + (UUID_LENGTH * 2) {
-            // second length is what we'd expect to see if the requested url is
-            // reflected in the response along with some static content; aka custom 404
-            let url_len = ferox_url.path_length()?;
-
-            wildcard.dynamic = wc_length - url_len;
-
-            if matches!(
-                self.handles.config.output_level,
-                OutputLevel::Default | OutputLevel::Quiet
-            ) {
-                let msg = format_template!("{} {:>9} {:>9} {:>9} Wildcard response is dynamic; {} ({} + url length) responses; toggle this behavior by using {}\n", wildcard.dynamic);
-                ferox_print(&msg, &PROGRESS_PRINTER);
-            }
-        } else if wc_length == wc2_length {
-            wildcard.size = wc_length;
-
-            if matches!(
-                self.handles.config.output_level,
-                OutputLevel::Default | OutputLevel::Quiet
-            ) {
-                let msg = format_template!("{} {:>9} {:>9} {:>9} Wildcard response is static; {} {} responses; toggle this behavior by using {}\n", wildcard.size);
-                ferox_print(&msg, &PROGRESS_PRINTER);
-            }
-        }
-
-        self.send_filter(wildcard)?;
-
         log::trace!("exit: wildcard_test");
         Ok(2)
     }
@@ -151,14 +166,30 @@ impl HeuristicTests {
     async fn make_wildcard_request(
         &self,
         target: &FeroxUrl,
+        method: &str,
+        data: Option<&[u8]>,
         length: usize,
     ) -> Result<FeroxResponse> {
         log::trace!("enter: make_wildcard_request({}, {})", target, length);
 
         let unique_str = self.unique_string(length);
-        let nonexistent_url = target.format(&unique_str, None)?;
 
-        let response = logged_request(&nonexistent_url.to_owned(), self.handles.clone()).await?;
+        // To take care of slash when needed
+        let slash = if self.handles.config.add_slash {
+            Some("/")
+        } else {
+            None
+        };
+
+        let nonexistent_url = target.format(&unique_str, slash)?;
+
+        let response = logged_request(
+            &nonexistent_url.to_owned(),
+            method,
+            data,
+            self.handles.clone(),
+        )
+        .await?;
 
         if self
             .handles
@@ -167,8 +198,14 @@ impl HeuristicTests {
             .contains(&response.status().as_u16())
         {
             // found a wildcard response
-            let mut ferox_response =
-                FeroxResponse::from(response, true, self.handles.config.output_level).await;
+            let mut ferox_response = FeroxResponse::from(
+                response,
+                &target.target,
+                method,
+                true,
+                self.handles.config.output_level,
+            )
+            .await;
             ferox_response.set_wildcard(true);
 
             if self
@@ -210,7 +247,7 @@ impl HeuristicTests {
             let url = FeroxUrl::from_string(target_url, self.handles.clone());
             let request = skip_fail!(url.format("", None));
 
-            let result = logged_request(&request, self.handles.clone()).await;
+            let result = logged_request(&request, DEFAULT_METHOD, None, self.handles.clone()).await;
 
             match result {
                 Ok(_) => {
