@@ -4,6 +4,7 @@ use crate::{
     config::OutputLevel,
     progress::PROGRESS_PRINTER,
     progress::{add_bar, BarType},
+    scan_manager::{MenuCmd, MenuCmdResult},
     scanner::RESPONSES,
     traits::FeroxSerialize,
     SLEEP_DURATION,
@@ -300,23 +301,33 @@ impl FeroxScans {
     }
 
     /// CLI menu that allows for interactive cancellation of recursed-into directories
-    async fn interactive_menu(&self) -> usize {
+    async fn interactive_menu(&self) -> Option<MenuCmdResult> {
         self.menu.hide_progress_bars();
         self.menu.clear_screen();
         self.menu.print_header();
         self.display_scans().await;
         self.menu.print_footer();
 
-        let mut num_cancelled = 0_usize;
+        let menu_cmd = if let Ok(line) = self.menu.term.read_line() {
+            self.menu.get_command_input_from_user(&line)
+        } else {
+            None
+        };
 
-        if let Some((input, force)) = self.menu.get_scans_from_user() {
-            num_cancelled += self.cancel_scans(input, force).await;
+        let result = match menu_cmd {
+            Some(MenuCmd::Cancel(indices, should_force)) => {
+                // cancel the things
+                let num_cancelled = self.cancel_scans(indices, should_force).await;
+                Some(MenuCmdResult::NumCancelled(num_cancelled))
+            }
+            Some(MenuCmd::Add(url)) => Some(MenuCmdResult::Url(url)),
+            None => None,
         };
 
         self.menu.clear_screen();
         self.menu.show_progress_bars();
 
-        num_cancelled
+        result
     }
 
     /// prints all known responses that the scanner has already seen
@@ -364,19 +375,19 @@ impl FeroxScans {
     ///
     /// When the value stored in `PAUSE_SCAN` becomes `false`, the function returns, exiting the busy
     /// loop
-    pub async fn pause(&self, get_user_input: bool) -> usize {
+    pub async fn pause(&self, get_user_input: bool) -> Option<MenuCmdResult> {
         // function uses tokio::time, not std
 
         // local testing showed a pretty slow increase (less than linear) in CPU usage as # of
         // concurrent scans rose when SLEEP_DURATION was set to 500, using that as the default for now
         let mut interval = time::interval(time::Duration::from_millis(SLEEP_DURATION));
-        let mut num_cancelled = 0_usize;
+        let mut command_result = None;
 
         if INTERACTIVE_BARRIER.load(Ordering::Relaxed) == 0 {
             INTERACTIVE_BARRIER.fetch_add(1, Ordering::Relaxed);
 
             if get_user_input {
-                num_cancelled += self.interactive_menu().await;
+                command_result = self.interactive_menu().await;
                 PAUSE_SCAN.store(false, Ordering::Relaxed);
                 self.print_known_responses();
             }
@@ -393,8 +404,8 @@ impl FeroxScans {
                     INTERACTIVE_BARRIER.fetch_sub(1, Ordering::Relaxed);
                 }
 
-                log::trace!("exit: pause_scan -> {}", num_cancelled);
-                return num_cancelled;
+                log::trace!("exit: pause_scan -> {:?}", command_result);
+                return command_result;
             }
         }
     }
