@@ -1,6 +1,7 @@
 use std::{ops::Deref, sync::atomic::Ordering, sync::Arc, time::Instant};
 
 use anyhow::{bail, Result};
+use console::style;
 use futures::{stream, StreamExt};
 use lazy_static::lazy_static;
 use tokio::sync::Semaphore;
@@ -10,14 +11,15 @@ use crate::{
         Command::{AddError, AddToF64Field, SubtractFromUsizeField},
         Handles,
     },
-    extractor::{ExtractionTarget::RobotsTxt, ExtractorBuilder},
+    extractor::{ExtractionTarget, ExtractorBuilder},
     heuristics,
-    scan_manager::{FeroxResponses, ScanOrder, ScanStatus, PAUSE_SCAN},
+    scan_manager::{FeroxResponses, MenuCmdResult, ScanOrder, ScanStatus, PAUSE_SCAN},
     statistics::{
         StatError::Other,
         StatField::{DirScanTimes, TotalExpected},
     },
     utils::fmt_err,
+    Command,
 };
 
 use super::requester::Requester;
@@ -42,7 +44,7 @@ pub struct FeroxScanner {
     /// wordlist that's already been read from disk
     wordlist: Arc<Vec<String>>,
 
-    /// limiter that restricts the number of active FeroxScanners  
+    /// limiter that restricts the number of active FeroxScanners
     scan_limiter: Arc<Semaphore>,
 }
 
@@ -72,23 +74,34 @@ impl FeroxScanner {
         log::trace!("enter: scan_url");
         log::info!("Starting scan against: {}", self.target_url);
 
-        let scan_timer = Instant::now();
+        let mut scan_timer = Instant::now();
+        let mut dirlist_flag = false;
 
-        if matches!(self.order, ScanOrder::Initial) && self.handles.config.extract_links {
-            // only grab robots.txt on the initial scan_url calls. all fresh dirs will be passed
-            // to try_recursion
+        if self.handles.config.extract_links {
+            // parse html for links (i.e. web scraping)
             let extractor = ExtractorBuilder::default()
+                .target(ExtractionTarget::ParseHtml)
                 .url(&self.target_url)
                 .handles(self.handles.clone())
-                .target(RobotsTxt)
                 .build()?;
-
-            let links = extractor.extract().await?;
+            let extract_out = extractor.extract().await?;
+            let links = extract_out.0;
+            dirlist_flag = extract_out.1;
             extractor.request_links(links).await?;
+
+            if matches!(self.order, ScanOrder::Initial) {
+                // check for robots.txt (cannot be in subdirs)
+                let extractor = ExtractorBuilder::default()
+                    .target(ExtractionTarget::RobotsTxt)
+                    .url(&self.target_url)
+                    .handles(self.handles.clone())
+                    .build()?;
+                let links = (extractor.extract().await?).0;
+                extractor.request_links(links).await?;
+            }
         }
 
         let scanned_urls = self.handles.ferox_scans()?;
-
         let ferox_scan = match scanned_urls.get_scan_by_url(&self.target_url) {
             Some(scan) => {
                 scan.set_status(ScanStatus::Running)?;
@@ -105,11 +118,40 @@ impl FeroxScanner {
 
         let progress_bar = ferox_scan.progress_bar();
 
+        // Directory listing heuristic detection to not continue scanning
+        if dirlist_flag {
+            log::trace!("exit: scan_url -> Directory listing heuristic");
+
+            self.handles.stats.send(AddToF64Field(
+                DirScanTimes,
+                scan_timer.elapsed().as_secs_f64(),
+            ))?;
+
+            self.handles.stats.send(SubtractFromUsizeField(
+                TotalExpected,
+                progress_bar.length() as usize,
+            ))?;
+
+            progress_bar.reset_eta();
+            progress_bar.finish_with_message(&format!(
+                "=> {}",
+                style("Directory listing").blue().bright()
+            ));
+
+            ferox_scan.finish()?;
+
+            return Ok(());
+        }
+
         // When acquire is called and the semaphore has remaining permits, the function immediately
         // returns a permit. However, if no remaining permits are available, acquire (asynchronously)
         // waits until an outstanding permit is dropped, at which point, the freed permit is assigned
         // to the caller.
         let _permit = self.scan_limiter.acquire().await;
+        if self.handles.config.scan_limit > 0 {
+            scan_timer = Instant::now();
+            progress_bar.reset();
+        }
 
         // Arc clones to be passed around to the various scans
         let looping_words = self.wordlist.clone();
@@ -122,7 +164,8 @@ impl FeroxScanner {
         }
 
         let requester = Arc::new(Requester::from(self, ferox_scan.clone())?);
-        let increment_len = (self.handles.config.extensions.len() + 1) as u64;
+        let increment_len =
+            ((self.handles.config.extensions.len() + 1) * self.handles.config.methods.len()) as u64;
 
         // producer tasks (mp of mpsc); responsible for making requests
         let producers = stream::iter(looping_words.deref().to_owned())
@@ -137,14 +180,30 @@ impl FeroxScanner {
                             // for every word in the wordlist, check to see if PAUSE_SCAN is set to true
                             // when true; enter a busy loop that only exits by setting PAUSE_SCAN back
                             // to false
-                            let num_cancelled = scanned_urls_clone.pause(true).await;
-                            if num_cancelled > 0 {
-                                handles_clone
-                                    .stats
-                                    .send(SubtractFromUsizeField(TotalExpected, num_cancelled))
-                                    .unwrap_or_else(|e| {
-                                        log::warn!("Could not update overall scan bar: {}", e)
-                                    });
+                            match scanned_urls_clone.pause(true).await {
+                                Some(MenuCmdResult::Url(url)) => {
+                                    // user wants to add a new url to be scanned, need to send
+                                    // it over to the event handler for processing
+                                    handles_clone
+                                        .send_scan_command(Command::ScanNewUrl(url))
+                                        .unwrap_or_else(|e| {
+                                            log::warn!("Could not add scan to scan queue: {}", e)
+                                        })
+                                }
+                                Some(MenuCmdResult::NumCancelled(num_canx)) => {
+                                    if num_canx > 0 {
+                                        handles_clone
+                                            .stats
+                                            .send(SubtractFromUsizeField(TotalExpected, num_canx))
+                                            .unwrap_or_else(|e| {
+                                                log::warn!(
+                                                    "Could not update overall scan bar: {}",
+                                                    e
+                                                )
+                                            });
+                                    }
+                                }
+                                _ => {}
                             }
                         }
                         requester_clone
